'use strict';

var _require = require('retry-task'),
    delayedRetry = _require.delayedRetry;

var request = require('../utils/request');
var getSession = require('./session-store');

var _require2 = require('./url-builder'),
    buildWatchFragmentsUrl = _require2.buildWatchFragmentsUrl,
    buildCommentServiceUrl = _require2.buildCommentServiceUrl;

// TODO: make # retries configurable


var withRetries = delayedRetry(3, function (n) {
  return n * n * 500;
});

var requestWithRetries = function requestWithRetries(arg) {
  return withRetries(function () {
    return request(arg);
  });
};

var buildRequestForm = function buildRequestForm(session, pageToken) {
  return {
    session_token: session.sessionToken,
    page_token: pageToken
  };
};

var buildJsonPostRequest = function buildJsonPostRequest(url, form) {
  return {
    method: 'POST',
    headers: {
      'accept-language': 'en-US;q=1.0,en;q=0.9'
    },
    json: true,
    url: url,
    form: form };
};

var commentPage = function commentPage(videoId, pageToken) {
  return getSession(videoId).map(function (sess) {
    return buildJsonPostRequest(buildCommentServiceUrl('action_get_comments'), buildRequestForm(sess, pageToken));
  }).chain(requestWithRetries);
};

var commentReplies = function commentReplies(videoId, repliesToken) {
  return getSession(videoId).map(function (sess) {
    return buildJsonPostRequest(buildCommentServiceUrl('action_get_comment_replies'), buildRequestForm(sess, repliesToken));
  }).chain(requestWithRetries);
};

var commentsWatchFragment = function commentsWatchFragment(videoId) {
  return getSession(videoId).map(function (sess) {
    return buildJsonPostRequest(buildWatchFragmentsUrl(videoId, sess, ['comments']), buildRequestForm(sess));
  }).chain(requestWithRetries);
};

module.exports = { commentPage: commentPage, commentReplies: commentReplies, commentsWatchFragment: commentsWatchFragment };