'use strict';

var _require = require('chai'),
    expect = _require.expect;

var cheerio = require('cheerio');

var fetchComments = require('../../lib/fetch-comments');

var validateComment = function validateComment(c) {
  expect(c).to.have.property('id').that.is.a('string').of.length.at.least(1);
  expect(c).to.have.property('author').that.is.a('string').of.length.at.least(1);
  expect(c).to.have.property('authorLink').that.is.a('string').of.length.at.least(1);
  expect(c).to.have.property('authorThumb').that.is.a('string').of.length.at.least(1);
  expect(c).to.have.property('text').that.is.a('string').of.length.at.least(1);
  expect(c).to.have.property('likes').that.is.a('number').of.at.least(0);
  expect(c).to.have.property('time').that.is.a('string').of.length.at.least(1);
  expect(c).to.have.property('timestamp').that.is.a('number').above(0), expect(c).to.have.property('edited').that.is.a('boolean');
};

describe('/lib/comment-stream', function () {
  this.timeout(30000);

  it('fetches first page of comments (no pageToken)', function (done) {
    var videoId = '9bZkp7q19f0';
    fetchComments(videoId).fork(function (e) {
      return done('Got an error: ' + e);
    }, function (p) {
      expect(p).to.have.property('comments').that.is.an('array').of.length.above(1);
      expect(p).to.have.property('nextPageToken').that.is.a('string').of.length.above(1);

      p.comments.forEach(function (c) {
        validateComment(c);
        expect(c).to.have.property('hasReplies').that.is.a('boolean');
        if (c.hasReplies) {
          expect(c).to.have.property('replies').that.is.an('array').of.length(c.numReplies);
          expect(c).to.have.property('numReplies').that.is.a('number').that.is.equal(c.replies.length);
          c.replies.forEach(validateComment);
        }
      });
      done();
    });
  });

  it('fetches next page of comments (with pageToken)', function (done) {
    var videoId = '9bZkp7q19f0';
    fetchComments(videoId).chain(function (_ref) {
      var nextPageToken = _ref.nextPageToken;
      return fetchComments(videoId, nextPageToken);
    }).fork(function (e) {
      return done('Got an error: ' + e);
    }, function (p) {
      expect(p).to.have.property('comments').that.is.an('array').of.length.above(1);
      expect(p).to.have.property('nextPageToken').that.is.a('string').of.length.above(1);

      p.comments.forEach(function (c) {
        validateComment(c);
        expect(c).to.have.property('hasReplies').that.is.a('boolean');
        if (c.hasReplies) {
          expect(c).to.have.property('replies').that.is.an('array').of.length(c.numReplies);
          expect(c).to.have.property('numReplies').that.is.a('number').that.is.equal(c.replies.length);
          c.replies.forEach(validateComment);
        }
      });
      done();
    });
  });
});